@@ -1,6 +1,6 @@
 /* The MIT License
 
-    Copyright (C) 2020-2021 Giulio Genovese
+    Copyright (C) 2020-2023 Giulio Genovese
 
     Author: Giulio Genovese <giulio.genovese@gmail.com>
 
@@ -39,6 +39,7 @@ typedef struct
 {
     htsFile *fh;        // output file handle
     char *fname;        // output file name
+    char *index_fn;
 }
 subset_t;
 
@@ -59,6 +60,8 @@ typedef struct
     int record_cmd_line;
     char **hts_opts;
     int nhts_opts;
+    bcf_hdr_t *hdr;
+    int write_index;
 }
 args_t;
 
@@ -94,6 +97,7 @@ static const char *usage_text(void)
         "   -x, --extra STRING              Output records not overlapping listed regions in separate file\n"
         "   -p, --prefix STRING             Prepend string to output VCF names\n"
         "       --hts-opts LIST             Low-level options to pass to HTSlib, e.g. block_size=32768\n"
+        "       --write-index               Automatically index the output files [off]\n"
         "\n"
         "Examples:\n"
         "   # Scatter a VCF file by shards with 10000 variants each\n"
@@ -193,9 +197,13 @@ static void open_set(subset_t *set, args_t *args)
         if ( hts_opt_apply(set->fh, opts) ) error("Could not apply the HTS options\n");
         hts_opt_free(opts);
     }
-    bcf_hdr_t *hdr = bcf_sr_get_header(args->sr, 0);
-    if ( bcf_hdr_write(set->fh, hdr)!=0 ) error("[%s] Error: cannot write the header to %s\n", __func__, args->str.s);
-    if (args->record_cmd_line) bcf_hdr_append_version(hdr, args->argc, args->argv, "bcftools_plugin");
+    if ( !args->hdr )
+    {
+        args->hdr = bcf_sr_get_header(args->sr, 0);
+        if ( args->record_cmd_line ) bcf_hdr_append_version(args->hdr, args->argc, args->argv, "bcftools_plugin");
+    }
+    if ( bcf_hdr_write(set->fh, args->hdr)!=0 ) error("[%s] Error: cannot write the header to %s\n", __func__, args->str.s);
+    if ( args->write_index && init_index(set->fh,args->hdr,args->str.s,&set->index_fn)<0 ) error("Error: failed to initialise index for %s\n",args->str.s);
 }
 
 static void init_data(args_t *args)
@@ -256,7 +264,17 @@ static void destroy_data(args_t *args)
     for (i=0; i<args->nsets; i++)
     {
         subset_t *set = &args->sets[i];
-        if (set->fname) {
+        if (set->fname)
+        {
+            if ( args->write_index )
+            {
+                if ( bcf_idx_save(set->fh)<0 )
+                {
+                    if ( hts_close(set->fh)!=0 ) error("Error: close failed .. %s\n", set->fname);
+                    error("Error: cannot write to index %s\n", set->index_fn);
+                }
+                free(set->index_fn);
+            }
             if ( hts_close(set->fh)!=0 ) error("Error: close failed .. %s\n", set->fname);
             free(set->fname);
         }
@@ -270,7 +288,6 @@ static void destroy_data(args_t *args)
 
 static void process(args_t *args)
 {
-    bcf_hdr_t *hdr = bcf_sr_get_header(args->sr, 0);
     bcf1_t *rec = bcf_sr_get_line(args->sr, 0);
     subset_t *set;
 
@@ -283,7 +300,7 @@ static void process(args_t *args)
             open_set(set, args);
             args->rec_cnt = 0;
         }
-        if ( bcf_write(set->fh, hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
+        if ( bcf_write(set->fh, args->hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
         args->rec_cnt++;
         if (args->rec_cnt == args->nsites) {
           args->rec_cnt = 0;
@@ -293,15 +310,15 @@ static void process(args_t *args)
           args->chunk_cnt++;
         }
     } else {
-        if ( regidx_overlap(args->reg_idx, bcf_hdr_id2name(hdr, rec->rid), rec->pos, rec->pos, args->reg_itr) ) {
+        if ( regidx_overlap(args->reg_idx, bcf_hdr_id2name(args->hdr, rec->rid), rec->pos, rec->pos, args->reg_itr) ) {
             while (regitr_overlap(args->reg_itr)) {
                 int idx = regitr_payload(args->reg_itr, int);
                 set = &args->sets[idx];
-                if ( bcf_write(set->fh, hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
+                if ( bcf_write(set->fh, args->hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
             }
         } else if (args->extra) {
             set = &args->sets[args->nsets-1];
-            if ( bcf_write(set->fh, hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
+            if ( bcf_write(set->fh, args->hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
         }
     }
 }
@@ -335,6 +352,7 @@ int run(int argc, char **argv)
         {"extra",required_argument,NULL,'x'},
         {"prefix",required_argument,NULL,'p'},
         {"hts-opts",required_argument,NULL,5},
+        {"write-index",no_argument,NULL,6},
         {NULL,0,NULL,0}
     };
     int c;
@@ -392,6 +410,7 @@ int run(int argc, char **argv)
             case 'x': args->extra = optarg;  break;
             case 'p': args->prefix = optarg;  break;
             case  5 : args->hts_opts = hts_readlist(optarg, 0, &args->nhts_opts); break;
+            case  6 : args->write_index = 1; break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
